#include "dataspace_server.h"
#include "ipc.h"
#include "pager.h"



Pager::Pager(PageMapper *mapper)
: _start(0), _size(0), _mapper(mapper)
{
    _mapper->attach();
}

void Pager::close()
{
    _mapper->detach();
}

/* Expose a region of the file. */

long Pager::mmap(size_t position, size_t length, size_t *start_pos, size_t *end_pos, size_t *data_end)
{
    _start = trunc(position, PAGE_SIZE);
    _size = round(position + length, PAGE_SIZE) - _start;

    *start_pos = _start;
    *end_pos = _start + _size;
    *data_end = 0;

    return L4_EOK;
}

/* Map a flexpage corresponding to the dataspace 'offset' involving a 'hot_spot'
   (flexpage offset). */

long Pager::map(unsigned long offset, l4_addr_t hot_spot, unsigned long flags, l4_snd_fpage_t *region)
{
    offset_t file_offset = _start + offset;
    offset_t max_offset = _start + _size;
    Flexpage *flexpage = _mapper->get(file_offset, flags);

    /* Issue the flexpage via the IPC system. */

    long err = ipc_prepare_flexpage(flexpage, file_offset, max_offset, hot_spot, region);

    if (err)
      return err;

    err = complete_Dataspace_map(*region);

    if (err)
      return err;

    /* After the flexpage is issued, it is queued for future reuse. */

    _mapper->queue(flexpage);

    return IPC_MESSAGE_SENT;
}

/* Return the total size of the data. */

offset_t Pager::get_data_size()
{
    return _mapper->get_data_size();
}

// vim: tabstop=4 expandtab shiftwidth=4
